-- Declaracao da ENTIDADE (entity)
-- 	Ela estabelece a interface do sistema digital com o mundo, ou seja, os pinos (sinais) de entrada e saida.
-- 	O nome da ENTIDADE deve ter tamanho maximo de 32 caracteres, sem pontuacao ou caracteres especiais.
-- 	Deve comecar por letra e pode incluir algarismos numericos e 'underline' (_).
-- 	O VHDL nao distingue entre maiuscula e minuscula ('case insensitive')
-- 
entity ALU is
		-- Genericos
		-- Definem valores estaticos de desempenho ou de formacao
		-- A sintaxe eh: NOME_DO_GENERICO : TIPO (bit, boolean, character, integer, real, time, bit_vector, string) [:= valor_default];
	generic
	(
		WIDTH : integer := 4
						-- A ultima declaracao de generico nao tem ponto e virgula (;)
	);
	port
	(
		-- Portas
		-- A sintaxe eh: NOME_DA_PORTA : MODO (in, out, inout) TIPO (bit, boolean, character, integer, real, time, bit_vector, string);
		-- Entrada(s)
		A, B		: in  bit_vector(WIDTH-1 downto 0);	-- Signals A e B do MODO de entrada (in) e do TIPO bit_vector
		ALUControl	: in  bit_vector(2 downto 0);		-- Signal ALUControl do MODO de entrada (in) e do TIPO bit_vector

		-- Saida(s)
		Result		: out bit_vector(WIDTH-1 downto 0);	-- Signal Result do MODO de saida (out) e do TIPO bit_vector de tamanho WIDTH
		Zero		: out bit 							-- Signal Zero de MODO de saída (out) e do tipo bit
						-- A ultima declaracao de porta nao tem ponto e virgula (;)
	);
end ALU;

-- Declaracao da ARQUITETURA (architecture)
-- 	Ela estabelece a relacao entre os pinos de entrada e de saida, ou seja, o circuito a ser implementado.
-- 	O nome da ARQUITETURA deve ter tamanho maximo de 32 caracteres, sem pontuacao ou caracteres especiais.
-- 	Deve comecar por letra e pode incluir algarismos numericos e 'underline' (_).
-- 	Cada ENTIDADE pode ter mais de uma ARQUITETURA, assim e necessario distingui-las por meio de um nome.

architecture estrutural of ALU is	-- ARQUITETURA 'estrutural' associada aa ENTIDADE 'ALU'

	-- Funcao auxiliar para realizar a operação de soma
	function "+" (a, b: bit_vector) return bit_vector is
		variable s : bit_vector(a'range) := (others => '0');
		variable cin : bit := '0';
	begin
		abc: for i in a'reverse_range loop
			s(i) := a(i) xor b(i) xor cin;
			cin := (a(i) and cin) or ((a(i) or cin) and b(i));
		end loop abc;
		return s;
	end function "+";

	-- Funcao auxiliar para realizar a operação de subtração
	-- Utilizar a estrutura da função "+" para tipo bit_vector para implementar a funcao "-" para bit_vector,
	--considerando que uma subtração em binário pode ser expressa por:
	--                                     A - B = A + B"
	--sendo B" o complemento de 2 do valor de B.
	function "-" (a, b: bit_vector) return bit_vector is
	-- Complete a função
		variable s : bit_vector(a'range) := (others => '0');
		variable cin : bit := '1';
	begin
		abc: for i in a'reverse_range loop
			s(i) := a(i) xor not b(i) xor cin;
			cin := (a(i) and cin) or ((a(i) or cin) and not b(i));
		end loop abc;
		return s;
	end function "-";
	
	signal Sum : bit_vector(WIDTH-1 downto 0);
	signal ZeroExt : bit_vector(WIDTH-1 downto 0);
	signal Result_tmp : bit_vector(WIDTH-1 downto 0);

	constant Nullo : bit_vector(WIDTH-1 downto 0) := (others => '0');
	
	
	-- Inicio da declaracao da ARQUITETURA
begin
	-- Para a implementacao da arquitetura da ALU, recomenda-se o uso da construção concorrente "Selected Signal Assignment":
	--<optional_label>: with <expression> select
	--<target> <= <value> when <choices>
	--			  <value> when <choices>
	--			  <value> when <choices>
	-- 		      ...
	--			  <value> when others;
	--tendo como <target> Result (ou Result_tmp), e como <expression> ALUControl
	
	-- Dicas:
	-- Como o resultado da operacao SLT e a saida ZERO depende de Result, recomenda-se criar um sinal intermediario (Result_tmp)
	--que armazena o valor das operacoes aritmetica e lógicas, ou zero para as operacoes nao definidas. Lembre-se de que para a
	--operacao SLT eh necessario calcular A-B para verificar se A eh menor do que B.
	-- Depois, utiliza-se Result_tmp para definir o valor final de Result e de Zero.
	-- O valor de ZeroExt pode ser obtido por (0 => Result_tmp(WIDTH-1), others => '0')
	
	with ALUControl(0) select
		Sum <= A + B when '0',
				 A - B when '1';

	ZeroExt <= (0 => Sum(WIDTH-1), others => '0');
	
	with ALUControl select
		Result_tmp <= Sum When "000",
						  Sum when "001",
						  A and B when "010",
						  A or B when "011",
						  ZeroExt when "101",
						  Nullo when others;

	Zero <= '1' when Result_tmp = Nullo else '0';
--	with Result_tmp select
--		Zero <= '1' when Nullo,
--				  '0' when others;
		
	Result <= Result_tmp;

end estrutural;
