-- Declaracao da ENTIDADE (entity)
-- 	Ela estabelece a interface do sistema digital com o mundo, ou seja, os pinos (sinais) de entrada e saida.
-- 	O nome da ENTIDADE deve ter tamanho maximo de 32 caracteres, sem pontuacao ou caracteres especiais.
-- 	Deve comecar por letra e pode incluir algarismos numericos e 'underline' (_).
-- 	O VHDL nao distingue entre maiuscula e minuscula ('case insensitive')
-- 
entity FSM is
		-- Portas
		-- A sintaxe eh: NOME_DA_PORTA : MODO (in, out, inout) TIPO (bit, boolean, character, integer, real, time, bit_vector, string);
		-- Entrada(s)
	port
	(
		clk			: in  bit;				-- Signal CLK do MODO de entrada (in) e do TIPO bit
		reset			: in  bit;
		Zero			: in  bit;
		Op				: in  bit_vector(1 downto 0);				-- Signal Op do MODO de entrada (in) e do TIPO bit_vector
		Funct3		: in  bit_vector(2 downto 0);
		-- Saida(s)
		PCWrite		: out  bit;				-- Signal PCWrite do MODO de saida (out) e do TIPO bit
		AdrSrc		: out  bit;
		MemWrite		: out  bit;
		IRWrite		: out  bit;
		RegWrite		: out  bit;
		ResultSrc	: out  bit_vector(1 downto 0);
		ALUControl	: out  bit_vector(2 downto 0);
		ALUSrcA		: out  bit_vector(1 downto 0);
		ALUSrcB		: out  bit_vector(1 downto 0);
		ImmSrc		: out  bit_vector(1 downto 0);
		num_state	: out  bit_vector(3 downto 0);
		ZFlag_debug	: out  bit
						-- A ultima declaracao de porta nao tem ponto e virgula (;)
	);
end FSM;

-- Declaracao da ARQUITETURA (architecture)
-- 	Ela estabelece a relacao entre os pinos de entrada e de saida, ou seja, o circuito a ser implementado.
-- 	O nome da ARQUITETURA deve ter tamanho maximo de 32 caracteres, sem pontuacao ou caracteres especiais.
-- 	Deve comecar por letra e pode incluir algarismos numericos e 'underline' (_).
-- 	Cada ENTIDADE pode ter mais de uma ARQUITETURA, assim e necessario distingui-las por meio de um nome.

architecture estrutural of FSM is	-- ARQUITETURA 'estrutural' associada aa ENTIDADE 'ALU'

	-- Build an enumerated type for the state machine
	type State_machine is (S0, S1, S2, S3, S4, S5, S6, S7, S8, S9); 

	-- Register to hold the current state and the next state
	signal state, next_state   : State_machine;

	signal PCUpdate, Branch, ZFlag   : bit;
	
	-- Inicio da declaracao da ARQUITETURA
begin
	-- A Moore machine's outputs are dependent only on the current state.
	-- The output is written only when the state changes.  (State
	-- transitions are synchronous.)

	-- Logic to advance to the next state
	process (clk, reset)
	begin
		if reset = '1' then
			state <= S0;
		elsif (clk'event and clk='1') then
			state <= next_state;
		end if;
	end process;

	-- Logic to advance to the next state
	process (state)
	begin
		case state is
			when S0=>
				next_state <= S1;
			when S1=>
				if    (Op = "00") then			-- CRm type
					next_state <= S6;
				elsif (Op = "01") then			-- CIm type
					next_state <= S8;
				elsif (Op = "10") then			-- CLm/CSm type
					next_state <= S2;
				else--if (Op = "11") then		-- CBm type
					next_state <= S9;
				end if;
			when S2=>
				if (Funct3(0) = '0') then
					next_state <= S3;
				else
					next_state <= S5;
				end if;
			when S3 =>
					next_state <= S4;
			when S4 =>
					next_state <= S0;
			when S5 =>
					next_state <= S0;
			when S6 =>
					next_state <= S7;
					ZFlag <= Zero;
			when S7 =>
					next_state <= S0;
			when S8 =>
					next_state <= S7;
					ZFlag <= Zero;
			when S9 =>
					next_state <= S0;
		end case;
	end process;

	-- Output depends solely on the current state
	process (state)
	begin
		case state is
			when S0 =>
				PCUpdate		<= '1';
				Branch		<= '0';
				AdrSrc		<= '0';
				MemWrite		<= '0';
				IRWrite		<= '1';
				RegWrite		<= '0';
				ResultSrc	<= "10";
				ALUControl	<= "000";
				ALUSrcA		<= "00";
				ALUSrcB		<= "10";
				--ImmSrc		<= "00";
				num_state	<= "0000";
			when S1 =>
				PCUpdate		<= '0';
				Branch		<= '0';
				--AdrSrc		<= '0';
				MemWrite		<= '0';
				IRWrite		<= '0';
				RegWrite		<= '0';
				--ResultSrc	<= "10";
				--ALUControl	<= "000";
				--ALUSrcA		<= "00";
				--ALUSrcB		<= "10";
				--ImmSrc		<= "00";
				num_state	<= "0001";
			when S2 =>							--(op=10)
				PCUpdate		<= '0';
				Branch		<= '0';
				--AdrSrc		<= '0';
				MemWrite		<= '0';
				IRWrite		<= '0';
				RegWrite		<= '0';
				--ResultSrc	<= "10";
				ALUControl	<= "000";
				ALUSrcA		<= "01";
				ALUSrcB		<= "01";
				ImmSrc		<= "01";
				num_state	<= "0010";
			when S3 =>
				PCUpdate		<= '0';
				Branch		<= '0';
				AdrSrc		<= '1';
				MemWrite		<= '0';
				IRWrite		<= '0';
				RegWrite		<= '0';
				ResultSrc	<= "00";
				--ALUControl	<= "000";
				--ALUSrcA		<= "01";
				--ALUSrcB		<= "01";
				--ImmSrc		<= "01";
				num_state	<= "0011";
			when S4 =>
				PCUpdate		<= '0';
				Branch		<= '0';
				--AdrSrc		<= '0';
				MemWrite		<= '0';
				IRWrite		<= '0';
				RegWrite		<= '1';
				ResultSrc	<= "01";
				--ALUControl	<= "000";
				--ALUSrcA		<= "01";
				--ALUSrcB		<= "01";
				--ImmSrc		<= "01";
				num_state	<= "0100";
			when S5 =>
				PCUpdate		<= '0';
				Branch		<= '0';
				AdrSrc		<= '1';
				MemWrite		<= '1';
				IRWrite		<= '0';
				RegWrite		<= '0';
				ResultSrc	<= "00";
				--ALUControl	<= "000";
				--ALUSrcA		<= "01";
				--ALUSrcB		<= "01";
				--ImmSrc		<= "01";
				num_state	<= "0101";
			when S6 =>							--(op=00)
				PCUpdate		<= '0';
				Branch		<= '0';
				--AdrSrc		<= '0';
				MemWrite		<= '0';
				IRWrite		<= '0';
				RegWrite		<= '0';
				--ResultSrc	<= "00";
				ALUControl	<= Funct3;
				ALUSrcA		<= "10";
				ALUSrcB		<= "00";
				--ImmSrc		<= "01";
				num_state	<= "0110";
			when S7 =>
				PCUpdate		<= '0';
				Branch		<= '0';
				--AdrSrc		<= '0';
				MemWrite		<= '0';
				IRWrite		<= '0';
				RegWrite		<= '1';
				ResultSrc	<= "00";
				ALUControl	<= Funct3;
				--ALUSrcA		<= "10";
				--ALUSrcB		<= "00";
				--ImmSrc		<= "01";
				num_state	<= "0111";
			when S8 =>							--(op=01)
				PCUpdate		<= '0';
				Branch		<= '0';
				--AdrSrc		<= '0';
				MemWrite		<= '0';
				IRWrite		<= '0';
				RegWrite		<= '0';
				--ResultSrc	<= "10";
				ALUControl	<= "000";
				ALUSrcA		<= "10";
				ALUSrcB		<= "01";
				ImmSrc		<= "00";
				num_state	<= "1000";
			when S9 =>							--(op=11)
				PCUpdate		<= '0';
				Branch		<= '1';
				--AdrSrc		<= '0';
				MemWrite		<= '0';
				IRWrite		<= '0';
				RegWrite		<= '0';
				ResultSrc	<= "10";
				ALUControl	<= "000";
				ALUSrcA		<= "01";
				ALUSrcB		<= "01";
				ImmSrc		<= "10";
				num_state	<= "1001";
		end case;
	end process;

	PCWrite <= (not ZFlag and Branch) or PCUpdate;
	
	ZFlag_debug <= not ZFlag;
	
end estrutural;