-- Declaracao da ENTIDADE (entity)
-- 	Ela estabelece a interface do sistema digital com o mundo, ou seja, os pinos (sinais) de entrada e saida.
-- 	O nome da ENTIDADE deve ter tamanho maximo de 32 caracteres, sem pontuacao ou caracteres especiais.
-- 	Deve comecar por letra e pode incluir algarismos numericos e 'underline' (_).
-- 	O VHDL nao distingue entre maiuscula e minuscula ('case insensitive')
-- 
entity RegBank is
		-- Genericos
		-- Definem valores estaticos de desempenho ou de formacao
		-- A sintaxe eh: NOME_DO_GENERICO : TIPO (bit, boolean, character, integer, real, time, bit_vector, string) [:= valor_default];
	generic
	(
		WIDTH : integer := 4;
		SIZE: integer := 2
						-- A ultima declaracao de generico nao tem ponto e virgula (;)
	);
	port
	(
		-- Portas
		-- A sintaxe eh: NOME_DA_PORTA : MODO (in, out, inout) TIPO (bit, boolean, character, integer, real, time, bit_vector, string);
		-- Entrada(s)
		CLK, WE3		: in  bit;				-- Signals CLK e EN do MODO de entrada (in) e do TIPO bit
		A1, A2, A3	: in  bit_vector(SIZE-1 downto 0);	-- Signals A1, A2 e A3 do MODO de entrada (in) e do TIPO bit_vector
		WD3		: in  bit_vector(WIDTH-1 downto 0);	-- Signal WD3 do MODO de entrada (in) e do TIPO bit_vector

		-- Saida(s)
		RD1, RD2	: out bit_vector(WIDTH-1 downto 0)	-- Signals RD1 e RD2 do MODO de saida (out) e do TIPO bit_vector de tamanho WIDTH
						-- A ultima declaracao de porta nao tem ponto e virgula (;)
	);
end RegBank;

-- Declaracao da ARQUITETURA (architecture)
-- 	Ela estabelece a relacao entre os pinos de entrada e de saida, ou seja, o circuito a ser implementado.
-- 	O nome da ARQUITETURA deve ter tamanho maximo de 32 caracteres, sem pontuacao ou caracteres especiais.
-- 	Deve comecar por letra e pode incluir algarismos numericos e 'underline' (_).
-- 	Cada ENTIDADE pode ter mais de uma ARQUITETURA, assim e necessario distingui-las por meio de um nome.

architecture estrutural of RegBank is	-- ARQUITETURA 'estrutural' associada aa ENTIDADE 'ALU'

	-- Funcao auxiliar para converter binário para decimal
	function to_integer (arg: bit_vector) return natural is
		variable result: natural := 0;
	begin
		for i in arg'range loop
			result := result+result;
				if arg(i) = '1' then
					result := result + 1;
				end if;
			end loop;
		return result;
	end to_integer;
	
	type t_RegFile is array (2**SIZE - 1 downto 0) of bit_vector(WIDTH - 1 downto 0);
	
	signal Regfile : t_RegFile;
	
	-- Inicio da declaracao da ARQUITETURA
begin
	
	process (CLK, WE3)
	begin
		if (CLK'event and CLK = '1') then
			if (WE3 = '1') then
				if (to_integer(A3) = 0) then
					RegFile(to_integer(A3)) <= (others => '0');
				else
					RegFile(to_integer(A3)) <= WD3;
				end if;
			end if;
		end if;
	end process;
	
	RD1 <= RegFile(to_integer(A1));
	RD2 <= RegFile(to_integer(A2));
	
	
end estrutural;
