-- Declaracao da ENTIDADE (entity)
-- 	Ela estabelece a interface do sistema digital com o mundo, ou seja, os pinos (sinais) de entrada e saida.
-- 	O nome da ENTIDADE deve ter tamanho maximo de 32 caracteres, sem pontuacao ou caracteres especiais.
-- 	Deve comecar por letra e pode incluir algarismos numericos e 'underline' (_).
-- 	O VHDL nao distingue entre maiuscula e minuscula ('case insensitive')
-- 
entity counter is
		-- Genericos
		-- Definem valores estaticos de desempenho ou de formacao
		-- A sintaxe eh: NOME_DO_GENERICO : TIPO (bit, boolean, character, integer, real, time, bit_vector, string) [:= valor_default];
	generic
	(
		WIDTH : integer := 4
						-- A ultima declaracao de generico nao tem ponto e virgula (;)
	);
	port
	(
		-- Portas
		-- A sintaxe eh: NOME_DA_PORTA : MODO (in, out, inout) TIPO (bit, boolean, character, integer, real, time, bit_vector, string);
		-- Entrada(s)
		clk	: in  bit;	-- Signal clk do MODO de entrada (in) e do TIPO bit
		clr	: in  bit;	-- Signal clr do MODO de entrada (in) e do TIPO bit

		-- Saida(s)
		q : out bit_vector(WIDTH-1 downto 0);	-- Signal q do MODO de saida (out) e do TIPO bit_vector de tamanho variavel de acordo com WIDTH
		cout: out bit
						-- A ultima declaracao de porta nao tem ponto e virgula (;)
	);
end counter;

-- Declaracao da ARQUITETURA (architecture)
-- 	Ela estabelece a relacao entre os pinos de entrada e de saida, ou seja, o circuito a ser implementado.
-- 	O nome da ARQUITETURA deve ter tamanho maximo de 32 caracteres, sem pontuacao ou caracteres especiais.
-- 	Deve comecar por letra e pode incluir algarismos numericos e 'underline' (_).
-- 	Cada ENTIDADE pode ter mais de uma ARQUITETURA, assim e necessario distingui-las por meio de um nome.

architecture operadores of counter is	-- ARQUITETURA 'operadores' associada aa ENTIDADE 'counter'

	-- Funcao auxiliar para incrementar o contador
	function "+" (a, b: bit_vector) return bit_vector is
		variable s : bit_vector(a'range) := (others => '0');
		variable cin : bit := '0';
	begin
		abc: for i in a'reverse_range loop
			s(i) := a(i) xor b(i) xor cin;
			cin := (a(i) and b(i)) or (cin and (a(i) or b(i)));
		end loop abc;
		return s;
	end function "+";

	-- Inicio da declaracao da ARQUITETURA

begin
	-- Processo para a implementacao do contador
	count: process (clk)
		--Variavel auxiliar para gerar um vetor de tamanho igual a q com valor inicial igual a "00...00"
		variable zero : bit_vector(q'range) := (others => '0');
		--Variavel auxiliar para gerar um vetor de tamanho igual a q com valor inicial igual a "00...01"
		variable um : bit_vector(q'range) := (0 => '1', others => '0');
		--Variavel auxiliar para armazenar o valor de contagem, inicializado com o valor "00...00";
		variable count : bit_vector(q'range) := (others => '0');
	begin
		-- Verifica se a entrada clr foi para '1'
		if (clr = '1') then
			count := zero;
		-- Verifica se a entrada clk se alterou (event) e foi para '1'
		elsif (clk'event and clk = '1') then
			count := count + um;
		end if;
		--Transfere o valor de contagem para a saída q
		q <= count;
		cout <= count(q'high);
	end process count;
end operadores;
