-- Declaracao da ENTIDADE (entity)
-- 	Ela estabelece a interface do sistema digital com o mundo, ou seja, os pinos (sinais) de entrada e saida.
-- 	O nome da ENTIDADE deve ter tamanho maximo de 32 caracteres, sem pontuacao ou caracteres especiais.
-- 	Deve comecar por letra e pode incluir algarismos numericos e 'underline' (_).
-- 	O VHDL nao distingue entre maiuscula e minuscula ('case insensitive')
-- 
entity decod is
	port
	(
		-- Portas
		-- A sintaxe eh: NOME_DA_PORTA : MODO (in, out, inout) TIPO (bit, boolean, character, integer, real, time, bit_vector, string);
		-- Entrada(s)
		b3	: in  bit;	-- Signal b0 do MODO de entrada (in) e do TIPO bit
		b2	: in  bit;	-- Signal b1 do MODO de entrada (in) e do TIPO bit
		b1	: in  bit;	-- Signal b2 do MODO de entrada (in) e do TIPO bit
		b0	: in  bit;	-- Signal b3 do MODO de entrada (in) e do TIPO bit

		-- Saida(s)
		seg_a : out bit;	-- Signal seg_a do MODO de saida (out) e do TIPO bit
		seg_b : out bit;	-- Signal seg_b do MODO de saida (out) e do TIPO bit
		seg_c : out bit;	-- Signal seg_c do MODO de saida (out) e do TIPO bit
		seg_d : out bit;	-- Signal seg_d do MODO de saida (out) e do TIPO bit
		seg_e : out bit;	-- Signal seg_e do MODO de saida (out) e do TIPO bit
		seg_f : out bit;	-- Signal seg_f do MODO de saida (out) e do TIPO bit
		seg_g : out bit;	-- Signal seg_g do MODO de saida (out) e do TIPO bit
		seg_dp : out bit	-- Signal seg_dp do MODO de saida (out) e do TIPO bit
								-- A ultima declaracao de porta nao tem ponto e virgula (;)
	);
end decod;

-- Declaracao da ARQUITETURA (architecture)
-- 	Ela estabelece a relacao entre os pinos de entrada e de saida, ou seja, o circuito a ser implementado.
-- 	O nome da ARQUITETURA deve ter tamanho maximo de 32 caracteres, sem pontuacao ou caracteres especiais.
-- 	Deve comecar por letra e pode incluir algarismos numericos e 'underline' (_).
-- 	Cada ENTIDADE pode ter mais de uma ARQUITETURA, assim e necessario distingui-las por meio de um nome.

architecture operadores of decod is	-- ARQUITETURA 'operadores' associada aa ENTIDADE 'decod'

	-- Sinais auxiliares que receberão o complemento de b3, b2, b1 e b0
	
	signal nb3, nb2, nb1, nb0: bit;

	-- Inicio da declaracao da ARQUITETURA

begin

	-- A atribuicao do valor para cada saida eh realizada por meio do operador atribuicao de sinal (<=)
	nb3 <= not b3;
	nb2 <= not b2;
	nb1 <= not b1;
	nb0 <= not b0;

	-- Para cada uma das saidas sera atribuida uma expressao booleana baseada nos operadores logicos E (and) OU (or) e NAO (not).
	seg_a <= (b3 or b2 or b1 or nb0) and (b3 or nb2 or b1 or b0) and (nb3 or nb2 or b1 or nb0) and (nb3 or b2 or nb1 or nb0); -- Escrita na forma de produto de somas
	seg_b <= (nb3 and nb2) or (nb2 and nb0) or (nb3 and nb1 and nb0) or (nb3 and b1 and b0) or (b3 and nb1 and b0);
	seg_c <= (b3 or b2 or nb1 or b0) and (nb3 or nb2 or b1 or b0) and (nb3 or nb2 or nb1);
	seg_d <= (b3 or b2 or b1 or nb0) and (b3 or nb2 or b1 or b0) and (nb3 or b2 or nb1 or b0) and (nb2 or nb1 or nb0);
	seg_e <= (nb2 and nb0) or (b3 and b2) or (b1 and nb0) or (b3 and b1);
	seg_f <= (nb1 and nb0) or (b3 and nb2) or (b3 and b1) or (nb3 and b2 and nb1) or (nb3 and b2 and nb0);
	seg_g <= (b3 or b2 or b1) and (nb3 or nb2 or b1 or b0) and (b3 or nb2 or nb1 or nb0);
	seg_dp <= '0';	-- Sempre desligado;
	
end operadores;
